@@ -2,12 +2,16 @@
 import os
 import sys
 import unittest
+import httmock
 
 sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
 import clever
 from clever import importer
 json = importer.import_json()
 
+import requests
+from httmock import response, HTTMock
+
 
 def functional_test(auth):
   class FunctionalTests(CleverTestCase):
@@ -119,12 +123,33 @@ class InvalidRequestErrorTest(CleverTestCase):
       self.assertFalse(isinstance(e.json_body, dict))  # 404 does not have a body
       self.assertTrue(isinstance(e.http_body, str))
 
+#generates httmock responses for TooManyRequestsErrorTest
+def too_many_requests_content(url, request):
+  headers =  {
+    'X-Ratelimit-Bucket': 'all, none',
+    'X-Ratelimit-Limit' : '200, 1200',
+    'X-Ratelimit-Reset' : '135136, 31634',
+    'X-Ratelimit-Remaining' : '0, 0'
+  }
+  return response(429, "", headers, None, 5, None)
+
+class TooManyRequestsErrorTest(CleverTestCase):
+
+  def test_rate_limiter(self):
+    with HTTMock(too_many_requests_content):
+      r = requests.get('https://test.rate.limiting')
+      res = {'body': r.content, 'headers': r.headers, 'code': 429}
+      APIRequestor = clever.APIRequestor()
+      self.assertRaises(clever.TooManyRequestsError, lambda : APIRequestor.interpret_response(res))
+
 if __name__ == '__main__':
   suite = unittest.TestSuite()
   for TestClass in [
           functional_test({"api_key": "DEMO_KEY"}),
           functional_test({"token": "7f76343d50b9e956138169e8cbb4630bb887b18"}),
           AuthenticationErrorTest,
-          InvalidRequestErrorTest]:
+          InvalidRequestErrorTest,
+          TooManyRequestsErrorTest]:
     suite.addTest(unittest.TestLoader().loadTestsFromTestCase(TestClass))
   unittest.TextTestRunner(verbosity=2).run(suite)
+
